require "fi/dir"

module Fi
  RSpec.describe Dir do
    let(:root) { Dir.current }
    let(:hidden_files) { /^\./ }
    let(:testdir) { 'testdir' }

    it "can get current directory" do
      expect(Dir.current).to eq ::Dir.pwd
    end

    it "can get home directory" do
      expect(Dir.home).to eq ENV['HOME']
    end

    it "can determine whether a directory exist" do
      expect(Dir.exist? '.').to be true
      expect(Dir.exist? 'nope').to be false
    end

    it "can go back to the previous directory" do
      original_dir = Dir.current
      new_dir = Fi::Path.expand(Dir.current + '/../')
      Dir.change new_dir
      expect(Dir.current).to eq new_dir
      Dir.back
      expect(Dir.current).to eq original_dir
    end

    it "can rename a directory" do
      Dir.new 't'
      Dir.rename 't', 'temp'

      expect('temp').to be_directory
      expect('t').not_to be_directory

      Dir.delete! 'temp'
    end

    # specify "change current directory" do
    #   Dir.change Dir.home
    #   expect(Dir.current).to eq Dir.home
    #   Dir.change root
    # end

    describe ".list and .list!" do
      it "lists files in current directory by default" do
        expect(Dir.list).to eq Dir.list Dir.current
        expect(Dir.list!).to eq Dir.list! Dir.current
      end

      it "lists non-hidden files in a directory" do
        list = Dir.list '~/'
        expect(list).not_to be_empty
        expect(list).not_to include hidden_files
      end

      it "lists all files in a directory" do
        list = Dir.list! '~/'
        expect(list).not_to be_empty
        expect(list).to include hidden_files
      end
    end


    describe ".new" do
      after { Dir.delete! testdir }

      it "can create a directory" do
        dir = Dir.new testdir
        expect(testdir).to be_directory
      end

      it "can create nested directories" do
        Dir.new 'new/yet/another'

        expect('new').to be_directory
        expect('new/yet').to be_directory
        expect('new/yet/another').to be_directory

        Dir.delete! 'new'
      end

      it "can create multiply directories" do
        Dir.new testdir, "#{testdir}1"

        expect(testdir).to be_directory
        expect("#{testdir}1").to be_directory

        Dir.delete! "#{testdir}1"
      end

      it "does nothing when a directory alreads exists" do
        Dir.new testdir
        Dir.new testdir
        expect(testdir).to be_directory
      end
    end


    describe ".delete and .delete!" do
      specify "`.delete` deletes an empty directory" do
        Dir.new 'temp'
        expect('temp').to be_directory

        Dir.delete 'temp'
        expect('temp').not_to be_directory
      end

      specify "`.delete!` deletes a non-empty directory" do
        Dir.new 'temp'
        expect('temp').to be_directory

        Dir.delete! 'temp'
        expect('temp').not_to be_directory
      end
    end
  end
end
